import type { Config } from 'prettier';

const prettier: Config = {
  arrowParens: 'avoid',
  tabWidth: 2,
  printWidth: 90,
  semi: true,
  singleQuote: true,
  trailingComma: 'all',
  plugins: [],
};

export default function defineConfig({
  tailwind,
  ...config
}: Partial<Config> & {
  tailwind?: boolean;
}) {
  const result: Config = {
    ...prettier,
    ...config,
  };

  if (tailwind) {
    ensureHas(result.plugins!, 'prettier-plugin-tailwindcss');
    result.tailwindAttributes ??= ['css'];
    result.tailwindFunctions ??= ['tw'];
  }

  return result;
}

function ensureHas<T>(list: T[], item: T) {
  if (!list.includes(item)) {
    list.push(item);
  }
}
